#!/usr/bin/perl

#================================================================
# This program is intended to read the CCFW settings for NIST security
# and use them to update the IBM WebSphere Liberty Profile settings
# in its jvm.options file.
# 
#================================================================

use Carp qw (confess);
use strict;

use constant TRUE				   => 1 ;

#------------------------------------------
# the filenames are part of the design and runtime contract between
# the CCFW team and the K2 team
#-----------------------------------------
use constant SECURITYCONF_FILENAME => "/opt/ccfw/data/securityConf.properties" ;
use constant WEBUICIPHER_FILENAME  => "/opt/hsc/data/cipher/webuicipher.conf" ;
use constant SECURITY_DEFINITION_FILENAME  => "/opt/ibm/esa/esa.war/WEB-INF/bootstrap.properties" ;

#------------------------------------------
# the allowed values of the $SECURITY_COMPLIANCE_PROPNAME in the SECURITYCONF_FILENAME
#------------------------------------------
use constant COMPLIANCE_LEGACY	   	=> "legacy" ;
use constant COMPLIANCE_NIST		=> "nist_sp800_131a" ;

#------------------------------------------
# the allowed values of the property names in the WEBUICIPHER_FILENAME
#------------------------------------------
use constant TLSV12_CIPHER			=> "TLSV12_CIPHER" ;
use constant TLSV11_CIPHER		    => "TLSV11_CIPHER" ;

#------------------------------------------
# the allowed values of the protocol
#------------------------------------------
use constant TLS		=> "SSL_TLSv2" ;
use constant TLS1_1		=> "TLSv1.1" ;
use constant TLS1_2		=> "TLSv1.2" ;

#------------------------------------------
# variables shared across the subroutines and the main
#------------------------------------------
our @TLS1_1Ciphers;
our @TLS1_2Ciphers;

our $securityComplianceValue;
our $jsse2_nist_value;

#------------------------------------------
# the main routine, this is run first
#------------------------------------------
main:
{

	#--------------------------------------------------
	# reads the SECURITYCONF_FILENAME and parses it to set
	# the global variable $securityComplianceValue
	#--------------------------------------------------
	readCurrentSecurityConfigurationProperties();

	#--------------------------------------------------
	# reads the WEBUICIPHER_FILENAME and parses it to set
	# the list of ciphers being used by the HMC
	#--------------------------------------------------
	readWebUICipherList();

    #------------------------------------------
    # write the values back to the XML file
    #------------------------------------------
	updateLibertySecurityDefinition();

	exit 0;
}

#--------------------------------------------------
# reads the SECURITYCONF_FILENAME and parses it to set
# the global variable $securityComplianceValue
#--------------------------------------------------
sub readCurrentSecurityConfigurationProperties
{
	my $line;
	my $numPropertiesFound = 0;

	$securityComplianceValue = COMPLIANCE_LEGACY;		#--- initialize in case it is not in file

	#----------------------------------------------
	# make sure we have permission to read the desired file
	#-----------------------------------------------
	unless( (-f SECURITYCONF_FILENAME) and (-r SECURITYCONF_FILENAME))
	{
		$! = 101;
		confess "ERROR 101:  Unable to read file " . SECURITYCONF_FILENAME . ".\n" ;
	}	

	unless( open( PROPS_FILE, "<" . SECURITYCONF_FILENAME ) )
	{
		$! = 102;
		confess "ERROR 102: Failed to open file " . SECURITYCONF_FILENAME . " for read.\n" ;
	}

	#------------------------------------------------
	# read the file a line at a time
	#------------------------------------------------

	while (TRUE)
	{
		$line = <PROPS_FILE>;

		#---- check for unexpected EOF
		if ( ! $line )
		{
			last;
		}

		chomp( $line );

		#---------------------------------------
		# skip any commented lines
		#--------------------------------------

		if ( $line =~ /^\s*#/ )
		{
			next;
		}

		#---------------------------------------
		# look for the payload line and grab the value
		#--------------------------------------
		if ( $line =~ /^\s*SecurityCompliance\s*=/ )
		{
			my $p_name;
			my $p_val;
			($p_name,$p_val) = split( /=/, $line );
			if ( $p_val )
			{
				$securityComplianceValue = $p_val;
			}
			$numPropertiesFound = $numPropertiesFound + 1;	
		}

		#--------------------------------------
		# skip any other lines
		#--------------------------------------
	}
	close( PROPS_FILE);

	#--------------------------------------------
	# make sure we found just 1 property line
	#--------------------------------------------
	unless ( $numPropertiesFound eq 1 )
	{
		$! = 103;
		confess "ERROR 103: Expected 1 but found $numPropertiesFound lines holding property 'SecurityCompliance'.\n";
	}
	
	print STDOUT "#--- updateSecurityDefinition.pl: the value of \$(lshmc -r -F security) is '$securityComplianceValue'\n"
	
}

#--------------------------------------------------
# reads the WEBUICIPHER_FILENAME and parses it to set
# the list of ciphers being used by the HMC
#--------------------------------------------------
sub readWebUICipherList
{
	my $line;
	my $numPropertiesFound = 0;

	#----------------------------------------------
	# make sure we have permission to read the desired file
	#-----------------------------------------------
	unless( (-f  WEBUICIPHER_FILENAME) and (-r WEBUICIPHER_FILENAME))
	{
		$! = 101;
		confess "ERROR 101:  Unable to read file " . WEBUICIPHER_FILENAME . ".\n" ;
	}	

	unless( open( CIPHER_FILE, "<" . WEBUICIPHER_FILENAME ) )
	{
		$! = 102;
		confess "ERROR 102: Failed to open file " . WEBUICIPHER_FILENAME . " for read.\n" ;
	}

	#------------------------------------------------
	# read the file a line at a time
	#------------------------------------------------

	while (TRUE)
	{
		$line = <CIPHER_FILE>;

		#---- check for unexpected EOF
		if ( ! $line )
		{
			last;
		}

		chomp( $line );

		#---------------------------------------
		# skip any commented lines
		#--------------------------------------

		if ( $line =~ /^\s*#/ )
		{
			next;
		}

		#---------------------------------------
		# look for the payload line and grab the values
		#--------------------------------------
		my $tlsv12 = TLSV12_CIPHER;

		my $tlsv12 = qr/${tlsv12}/;

		if ( $line =~ /^\s*${tlsv12}\s*=/ )
		{
			my $p_name;
			my $p_val;
			($p_name,$p_val) = split( /=/, $line );
			$numPropertiesFound = $numPropertiesFound + 1;
			if ( $p_val )
			{
				@TLS1_2Ciphers = split( /,/, $p_val );
			}		
		}


		#--------------------------------------
		# skip any other lines
		#--------------------------------------
	}
	close( CIPHER_FILE);

	#--------------------------------------------
	# make sure we found just 1 property line
	#--------------------------------------------
	unless ( $numPropertiesFound eq 1 )
	{
		$! = 103;
		confess "ERROR 103: Expected 1 but found $numPropertiesFound lines holding cipher lists in file" . WEBUICIPHER_FILENAME . ".\n";
	}
	
    print STDOUT "#--- updateSecurityDefinition.pl: the value of \$(lshmcencr -c webui -t c) is '@TLS1_2Ciphers'\n"
}


#--------------------------------------------------
# reads the current pmcSecurityDefinition.xml file
# modifies two attributes in an XML stanza
# writes the file back out
#--------------------------------------------------
sub updateLibertySecurityDefinition
{
	my @originalFile;
	my @modifiedFile;
	my $line;
	my $propname;
	my $updatedLine;
	my $nistValue;

	#----------------------------------------------
	# make sure we have permission to read and to write the desired file
	#----------------------------------------------

	unless( (-f SECURITY_DEFINITION_FILENAME) and (-r SECURITY_DEFINITION_FILENAME))
	{
		$! = 110;
		confess "ERROR 110:  Unable to read file " . SECURITY_DEFINITION_FILENAME . ".\n" ;
	}	

	unless( -w SECURITY_DEFINITION_FILENAME)
	{
		$! = 111;
		confess "ERROR 111:  Unable to write file " . SECURITY_DEFINITION_FILENAME . ".\n" ;
	}	

	unless( open( XML_FILE, "<" . SECURITY_DEFINITION_FILENAME  ) )
	{
		$! = 112;
		confess "ERROR 112: Failed to open file " . SECURITY_DEFINITION_FILENAME . " for read.\n" ;
	}

	#-----------------------------------------------
	# load the whole file into one array (it's not very big)
	#-----------------------------------------------

	@originalFile = <XML_FILE> ;
	close(XML_FILE);

	#------------------------------------------------
	# define the two properties that we are replacing
	#------------------------------------------------

	my $SSL_PROTOCOL = "sslProtocol";
	my $SSL_PROTOCOL_REGEX = qr/${SSL_PROTOCOL}/;

	my $ENABLED_CIPHERS = "enabledCiphers" ;
	my $ENABLED_CIPHERS_REGEX = qr/${ENABLED_CIPHERS}/;

	#----------------------------------------------
	# define the expected protocol for the value of security that was found
	#---------------------------------------------

	my $protocol = TLS;
	if ( $securityComplianceValue eq COMPLIANCE_NIST )
	{
		$protocol = TLS1_2;
	}

	#----------------------------------------------
	# define the expected ciphers for the value of security that was found
	#---------------------------------------------

	my @ciphers = @TLS1_2Ciphers;

	my $cipherlist = join( ' ' , @ciphers);
	$cipherlist =~  s/^\s+//;
	$cipherlist =~ s/\s+$//;

	#-----------------------------------------------
	# iterate over the file, looking for either of our two lines
	# fail if we find <1 or >1 instances of either
	#----------------------------------------------

	my $numCipherUpdates = 0;
	my $numProtocolUpdates = 0;

	foreach $line (@originalFile)
	{
		if ( $line =~ /^\s*${ENABLED_CIPHERS_REGEX}\s*=/ )
		{
			$updatedLine = "${ENABLED_CIPHERS}=" . $cipherlist . "\n";
			push( @modifiedFile, ($updatedLine));
			$numCipherUpdates = $numCipherUpdates + 1;
		}
		elsif ( $line =~ /^\s*${SSL_PROTOCOL_REGEX}\s*=/ )
		{
			$updatedLine = "${SSL_PROTOCOL}=" . $protocol . "\n";
			push( @modifiedFile, ($updatedLine));
			$numProtocolUpdates = $numProtocolUpdates + 1;
		}
		else
		{
			push( @modifiedFile, ($line));
		}
	}

	unless( $numCipherUpdates eq 1 )
	{
		$! = 114;
		confess "ERROR 114: Expected 1 but had $numCipherUpdates of property '$ENABLED_CIPHERS'.\n";
	}

	unless( $numProtocolUpdates eq 1 )
	{
		$! = 115;
		confess "ERROR 115: Expected 1 but had $numProtocolUpdates of property '$SSL_PROTOCOL'.\n";
	}

	#------------------------------------------------
	# ok, we made the right changes, so write the file out
	#------------------------------------------------

	unless( open( NEW_XML_FILE, ">" . SECURITY_DEFINITION_FILENAME  ) )
	{
		$! = 113;
		confess "ERROR 113: Failed to open file " . SECURITY_DEFINITION_FILENAME . " for write.\n" ;
	}
	
	foreach $line (@modifiedFile)
	{
		print NEW_XML_FILE $line ;
	}
	close(NEW_XML_FILE);
	
	print STDOUT "#--- updateSecurityDefinition.pl: using SSL protocol '$protocol'\n";
	print STDOUT "#--- updateSecurityDefinition.pl: the value of \$(lshmcencr -c webui -t c) is :\n" . join( ',', @ciphers) . "\n";
	
}
